/*
 * @file ServiceBroker_Module_Im.c
 * Module Interface for Module ServiceBroker_Module_Im
 */

#include <stdio.h>
#include "ServiceBroker_Module_Im.h"

void setProvidedServiceAvailable(ServiceBroker_Module_Im__context *context)
{
   ServiceBroker_Module_Im_container__BrokeredAvailable_handle availableHandle;

   // Set the provided service available if either required service instance is available.
   ECOA__return_status status = ServiceBroker_Module_Im_container__BrokeredAvailable__get_write_access(context, &availableHandle);

   if (status == ECOA__return_status_OK || status == ECOA__return_status_DATA_NOT_INITIALIZED)
   {
      if (context->user.preferredAvailable == ECOA__TRUE || context->user.backupAvailable == ECOA__TRUE)
      {
         *(availableHandle.data) = ECOA__TRUE;
      }
      else
      {
         *(availableHandle.data) = ECOA__FALSE;
      }

      ECOA__return_status status = ServiceBroker_Module_Im_container__BrokeredAvailable__publish_write_access(context, &availableHandle);
   }
}

void ServiceBroker_Module_Im__INITIALIZE__received(ServiceBroker_Module_Im__context *context)
{
   /* User Code Here */
}

void ServiceBroker_Module_Im__START__received(ServiceBroker_Module_Im__context *context)
{
   ServiceBroker_Module_Im_container__PreferredAvailable_handle preferredAvailableHandle;
   ServiceBroker_Module_Im_container__BackupAvailable_handle backupAvailableHandle;
   ECOA__return_status status;

   // Check if either the preferred or backup services are available.

   status = ServiceBroker_Module_Im_container__PreferredAvailable__get_read_access(context, &preferredAvailableHandle);
   if (status == ECOA__return_status_OK)
   {
      context->user.preferredAvailable = *(preferredAvailableHandle.data);

      status = ServiceBroker_Module_Im_container__PreferredAvailable__release_read_access(context, &preferredAvailableHandle);
   }

   status = ServiceBroker_Module_Im_container__BackupAvailable__get_read_access(context, &backupAvailableHandle);
   if (status == ECOA__return_status_OK)
   {
      context->user.backupAvailable = *(backupAvailableHandle.data);

      status = ServiceBroker_Module_Im_container__BackupAvailable__release_read_access(context, &backupAvailableHandle);
   }

   setProvidedServiceAvailable(context);
}

void ServiceBroker_Module_Im__STOP__received(ServiceBroker_Module_Im__context *context)
{
   /* User Code Here */
}

void ServiceBroker_Module_Im__SHUTDOWN__received(ServiceBroker_Module_Im__context *context)
{
   /* User Code Here */
}

void ServiceBroker_Module_Im__PreferredAvailable__updated(ServiceBroker_Module_Im__context* context)
{
   ServiceBroker_Module_Im_container__PreferredAvailable_handle preferredAvailableHandle;
   ECOA__return_status status;

   // Check if either the preferred or backup services are available.

   status = ServiceBroker_Module_Im_container__PreferredAvailable__get_read_access(context, &preferredAvailableHandle);
   if (status == ECOA__return_status_OK)
   {
      context->user.preferredAvailable = *(preferredAvailableHandle.data);

      status = ServiceBroker_Module_Im_container__PreferredAvailable__release_read_access(context, &preferredAvailableHandle);
   }

   setProvidedServiceAvailable(context);
}

void ServiceBroker_Module_Im__BackupAvailable__updated(ServiceBroker_Module_Im__context* context)
{
   ServiceBroker_Module_Im_container__BackupAvailable_handle backupAvailableHandle;
   ECOA__return_status status;

   status = ServiceBroker_Module_Im_container__BackupAvailable__get_read_access(context, &backupAvailableHandle);
   if (status == ECOA__return_status_OK)
   {
      context->user.backupAvailable = *(backupAvailableHandle.data);

      status = ServiceBroker_Module_Im_container__BackupAvailable__release_read_access(context, &backupAvailableHandle);
   }

   setProvidedServiceAvailable(context);
}

void ServiceBroker_Module_Im__Request_for_Val__request_received
   (ServiceBroker_Module_Im__context* context,
    const ECOA__uint32 ID,
    const ECOA__global_time* time)
{
   ECOA__return_status return_status;
   example__value_type val;
   ECOA__log log;

   // Forward the request to the preferred if it's available.
   if (context->user.preferredAvailable)
   {
      log.current_size = sprintf((char *) &log.data, "***ServiceBrokered - using preferred -  val before request = %d", val);
      ServiceBroker_Module_Im_container__log_info(context, log);

      return_status = ServiceBroker_Module_Im_container__Request_Val_Preferred__request_sync(context, time, &val);

      log.current_size = sprintf((char *) &log.data, "***ServiceBrokered - using backup - val from response = %d", val);
      ServiceBroker_Module_Im_container__log_info(context, log);
   }
   else if (context->user.backupAvailable)
   {
      // otherwise if the backup is available, use that.
      log.current_size = sprintf((char *) &log.data, "***ServiceBrokered - using backup -  val before request = %d", val);
      ServiceBroker_Module_Im_container__log_info(context, log);

      return_status = ServiceBroker_Module_Im_container__Request_Val_Backup__request_sync(context, time, &val);

      log.current_size = sprintf((char *) &log.data, "***ServiceBrokered - using backup - val from response = %d", val);
      ServiceBroker_Module_Im_container__log_info(context, log);
   }

   // Always respond.
   return_status = ServiceBroker_Module_Im_container__Request_for_Val__response_send(context, ID, val);
}
